###############################################################################################
# R-code
# File: modelestimate_intercept.r
#
# This program is designed by H. Chen and T.L. Chong(2005) to estimate the AR 
# coefficients and the order  in each regime using AIC for the M-TAR model 
# with two threshold variables. 
#
# y      = the time series data.  
# Z1, Z2 = two vectors as the threshold variables.
#
# Searching area is between the (lowb,upb)=(0.15,0.85) rank of the sample.
#   Threshold = the 2 by 1 vector indicating the threshold values.
#   regime    = a 4 by 1 vector, which indicates the model structure in different regimes.  
#
# For details, c(1,2,3,4) means that the model is four-regime model; 
#              c(1,1,2,3) means that the model is three-regime model and the 
#                         regimes I and regime II should be combined together.  
# Similarly,   c(1,2,1,3) means that the model is three-regime model and the 
#                         regime I and regime III should be combined together. 
# Moreover,    c(1,1,1,2) means that the model is two-regime model 
#                         and regime I, II, III should be put together.
#     kmax = the maximum number of the order. 
#
# The function returns the final estimated coefficients of the model and the 
# estimated threshold values.
#
# Reference: 
# Chen, H., Chong, T.T.L., and Bai, J. (2012).
#    Theory and applications of TAR model with two threshold variables.
#    Econometric Reviews, 31(2), 142-170.
#    DOI: 10.1080/07474938.2011.607100.
###############################################################################################

modelest <- function(y,z1,z2,regime,threshold){

lz   = length(z1);
ly   = length(y);
kmax = 10;

if(lz!=ly){
   print("the size of the sample does not match");
   return(0);  
}

threshold1 = threshold[1];
threshold2 = threshold[2];

c1 = which(z1>=threshold1&z2>=threshold2);
c2 = which(z1<threshold1&z2>=threshold2);
c3 = which(z1<threshold1&z2<threshold2);
c4 = which(z1>=threshold1&z2<threshold2);

if(sum(regime==c(1,2,3,4))==4){
   minaic1 = minaic2 = minaic3 = minaic4 = 999999;
   aicm1   = aicm2 = aicm3 = aicm4 = rep(0,10);

   if(length(c1)>=3){
     for(k1 in 1:kmax){
       cc1 = c1[which(c1>k1)];
       y1  = y[cc1];
       x1  = y[cc1-1];
       if(k1>1){
         for(j in 2:k1){
           x1 = cbind(x1,y[cc1-j])
         }
       }
       
       a1        = coef(lm(y1~x1));
       re1       = sum(resid(lm(y1~x1))^2);
       resid1    = resid(lm(y1~x1));
       aicm1[k1] = aic1=length(y1)*log(re1/length(y1))+2*(k1+1);
       if(aic1<minaic1){
          minaic1 = aic1;
          minre1  = re1;
          mr1     = resid1;
          ma1     = a1;
          mcc1    = cc1;
        }
     } 
}  #  end loop if(sum(regime==c(1,2,3,4))==4)

if(length(c1)<3){minre1=ma1=mcc1=mink1=mr1=minaic1=0;}
                
if(length(c2)>=3){
  for(k2 in 1:kmax){ 
     cc2 = c2[which(c2>k2)];    
     y2  = y[cc2];
     x2  = y[cc2-1];
     if(k2>1){
       for(j in 2:k2){
         x2 = cbind(x2,y[cc2-j]);
       }
     }
     a2        = coef(lm(y2~x2));
     re2       = sum(resid(lm(y2~x2))^2);
     resid2    = resid(lm(y2~x2));
     aicm2[k2] = aic2=length(y2)*log(re2/length(y2))+2*(k2+1);
     if(aic2<minaic2){
        mr2     = resid2;
        minaic2 = aic2;
        minre2  = re2;
        ma2     = a2;
        mcc2    = cc2;
      }
   }
} # end loop if(length(c2)>=3)

if(length(c2)<3) {minre2=mcc2=ma2=mink2=mr2=minaic2=0;}

if(length(c3)>=3){
  for(k3 in 1:kmax){
    cc3 = c3[which(c3>k3)];
    y3  = y[cc3];
    x3  = y[cc3-1];
    if(k3>1){
       for(j in 2:k3){
         x3 = cbind(x3,y[cc3-j]);
       }
    }
    a3     = coef(lm(y3~x3));
    re3    = sum(resid(lm(y3~x3))^2);
    resid3 = resid(lm(y3~x3));
    aic3   = aicm3[k3]=length(y3)*log(re3/length(y3))+2*(k3+1);
    if(aic3<minaic3){
       mr3     = resid3;
       minaic3 = aic3;
       minre3  = re3;
       ma3     = a3;
       mcc3    = cc3;
    }
  }  # end loop for(k3 in 1:kmax)
}    # end loop if(length(c3)>=3)

if(length(c3)<3) {minre3=mcc3=mink3=ma3=mr3=minaic3=0;}

if(length(c4)>=3){
  for(k4 in 1:kmax){
    cc4=c4[which(c4>k4)];
    y4=y[cc4];
    x4=y[cc4-1];
    if(k4>1){
      for(j in 2:k4){
        x4=cbind(x4,y[cc4-j])
      }
    }
    a4     = coef(lm(y4~x4));
    resid4 = resid(lm(y4~x4));
    re4    = sum(resid(lm(y4~x4))^2); 
    aic4   = aicm4[k4]=length(y4)*log(re4/length(y4))+2*(k4+1);
    if(aic4<minaic4){
      minaic4 = aic4;
      mr4     = resid4;
      minre4  = re4;
      ma4     = a4;
      mcc4    = cc4;
    }
  } # end loop for(k4 in 1:kmax)
}   # end loop if(length(c4)>=3)

if(length(c4)<3){ minre4=mcc4=ma4=mr4=minaic4=0;}
  ta  = minaic1+minaic2+minaic3+minaic4;
  rss = minre1+minre2+minre3+minre4;
  result <- list(cc1=mcc1,cc2=mcc2,cc3=mcc3,cc4=mcc4,a1=ma1,a2=ma2,a3=ma3,a4=ma4,re1=mr1,re2=mr2,re3=mr3,re4=mr4,rss=rss,aic=ta);
  return(result);
}

if(sum(regime==c(1,1,2,3))==4){
  minaic12 = minaic3=minaic4=999999;
  aicm12   = aicm3=aicm4=rep(0,10);
  c12      = c(c1,c2);
  if(length(c12)>=3){
    for(k12 in 1:kmax){
      cc12 = c12[which(c12>k12)];
      y12  = y[cc12];
      x12  = y[cc12-1];
      if(k12>1){
        for(j in 2:k12){
          x12 = cbind(x12,y[cc12-j])
        }
      }
      a12         = coef(lm(y12~x12));
      re12        = sum(resid(lm(y12~x12))^2);
      resid12     = resid(lm(y12~x12));
      aicm12[k12] = aic12=length(y12)*log(re12/length(y12))+2*(k12+1);
      if(aic12<minaic12){
        minaic12 = aic12;
        minre12  = re12;
        mr12     = resid12;
        ma12     = a12;
        mcc12    = cc12;
       }
    }  # end loop if(length(c12)>=3)
}      # end loop if(sum(regime==c(1,1,2,3))==4)

if(length(c12)<3){ 
         minre12=mcc12=ma12=mink12=mr12=minaic12=0;}  
            
if(length(c3)>=3){
  for(k3 in 1:kmax){
    cc3 = c3[which(c3>k3)];
    y3  = y[cc3];
    x3  = y[cc3-1];
    if(k3>1){
      for(j in 2:k3){
        x3 = cbind(x3,y[cc3-j]);
      }
    }
    a3        = coef(lm(y3~x3));
    re3       = sum(resid(lm(y3~x3))^2);
    resid3    =resid(lm(y3~x3));
    aicm3[k3] = aic3=length(y3)*log(re3/length(y3))+2*(k3+1);
    if(aic3<minaic3){
      mr3     = resid3;
      minaic3 = aic3;
      minre3  = re3;
      ma3     = a3;
      mcc3    = cc3;
    }
  }
}    # end loop if(length(c3)>=3)
                               
if(length(c3)<3) { mr3=minre3=mcc3=mink3=ma3=minaic3=0;}

if(length(c4)>=3){
  for(k4 in 1:kmax){
    cc4 = c4[which(c4>k4)];
    y4  = y[cc4];
    x4  = y[cc4-1];
    if(k4>1){
      for(j in 2:k4){
        x4=cbind(x4,y[cc4-j])
      }
    }
    a4     = coef(lm(y4~x4));
    resid4 = resid(lm(y4~x4));
    re4    = sum(resid(lm(y4~x4))^2); 
    aic4   = aicm4[k4]=length(y4)*log(re4/length(y4))+2*(k4+1);
    if(aic4<minaic4){
      minaic4 = aic4;
      mr4     = resid4;
      minre4  = re4;
      ma4     = a4;
      mcc4    = cc4;
    }
  }
}    # end loop if(length(c4)>=3)

if(length(c4)<3){ mr4=minre4=mcc4=ma4=minaic4=0;}
  aic = minaic12+minaic3+minaic4;
  rss = minre12+minre3+minre4;
  result <- list(cc12=mcc12,cc3=mcc3,cc4=mcc4,a12=ma12,a3=ma3,a4=ma4,re12=mr12,re3=mr3,re4=mr4,rss=rss,aic=aic)
  return(result);
}

if(sum(regime==c(1,2,2,3))==4){
   minaic1 = minaic23=minaic4=999999;
   c23     = c(c2,c3);
   aicm1   = aicm23=aicm4=rep(0,10);
   if(length(c1)>=3){
     for(k1 in 1:kmax){
        cc1 = c1[which(c1>k1)];
        y1  = y[cc1];
        x1  = y[cc1-1];
        if(k1>1){
          for(j in 2:k1){
            x1 = cbind(x1,y[cc1-j])
          }
        }
        a1     = coef(lm(y1~x1));
        re1    = sum(resid(lm(y1~x1))^2);
        resid1 = resid(lm(y1~x1));
        aicm1[k1] = aic1=length(y1)*log(re1/length(y1))+2*(k1+1);
        if(aic1<minaic1){
          minaic1 = aic1;
          minre1  = re1;
          mr1     = resid1;
          ma1     = a1;
          mcc1    = cc1;
        }
    } 
}     # end loop if(sum(regime==c(1,2,2,3))==4)

if(length(c1)<3){ 
        mr1=minre1=ma1=mcc1=mink1=minaic1=0;
            }                  
if(length(c23)>=3){
  for(k23 in 1:kmax){ 
    cc23 = c23[which(c23>k23)];    
    y23  = y[cc23];
    x23  = y[cc23-1];
    if(k23>1){
      for(j in 2:k23){
        x23 = cbind(x23,y[cc23-j]);
      }
    }
    a23     = coef(lm(y23~x23));
    re23    = sum(resid(lm(y23~x23))^2);
    resid23 = resid(lm(y23~x23));
    aicm23[k23] = aic23=length(y23)*log(re23/length(y23))+2*(k23+1);
    if(aic23<minaic23){
      mr23     = resid23;
      minaic23 = aic23;
      minre23  = re23;
      ma23     = a23;
      mcc23    = cc23;
    }
  }
}     # end loop if(length(c23)>=3)

if(length(c23)<3) { mr23=minre23=mcc23=ma23=mink23=minaic23=0;}

if(length(c4)>=3){
  for(k4 in 1:kmax){
    cc4 = c4[which(c4>k4)];
    y4  = y[cc4];
    x4  = y[cc4-1];
    if(k4>1){
      for(j in 2:k4){
         x4 = cbind(x4,y[cc4-j])
      }
    }
    a4     = coef(lm(y4~x4));
    resid4 = resid(lm(y4~x4));
    re4    = sum(resid(lm(y4~x4))^2); 
    aic4   = aicm4[k4]=length(y4)*log(re4/length(y4))+2*(k4+1);
    if(aic4<minaic4){
      minaic4 = aic4;
      mr4     = resid4;
      minre4  = re4;
      ma4     = a4;
      mcc4    = cc4;
    }
  }
}     # end loop if(length(c4)>=3)

if(length(c4)<3){ mr4=minre4=mink4=mcc4=ma4=minaic4=0;}
  aic = minaic1+minaic23+minaic4;
  rss = minre1+minre23+minre4;
  result <- list(cc23=mcc23,cc1=mcc1,cc4=mcc4,a1=ma1,a23=ma23,a4=ma4,re1=mr1,re23=mr23,re4=mr4,rss=rss,aic=aic);
  return(result);
}

if(sum(regime==c(1,2,3,3))==4){
  minaic1 = minaic2=minaic34=999999;
  kmax    = 10;
  c34     = c(c3,c4);
  aicm1   = aicm2=aicm34=rep(0,10);
  if(length(c1)>=3){
     for(k1 in 1:kmax){
        cc1 = c1[which(c1>k1)];
        y1  = y[cc1];
        x1  = y[cc1-1];
        if(k1>1){
          for(j in 2:k1){
             x1 = cbind(x1,y[cc1-j])
          }
        }
        a1     = coef(lm(y1~x1));
        re1    = sum(resid(lm(y1~x1))^2);
        resid1 = resid(lm(y1~x1));
        aicm1[k1] = aic1=length(y1)*log(re1/length(y1))+2*(k1+1);
        if(aic1<minaic1){
          minaic1 = aic1;
          minre1  = re1;
          mr1     = resid1;
          ma1     = a1;
          mcc1    = cc1;
         }
      } 
}    #  end loop if(sum(regime==c(1,2,3,3))==4)

if(length(c1)<3){mr1=minre1=mcc1=ma1=mink1=minaic1=0;}                  
 
if(length(c2)>=3){
  for(k2 in 1:kmax){ 
     cc2 = c2[which(c2>k2)];    
     y2  = y[cc2];
     x2  = y[cc2-1];
     if(k2>1){
       for(j in 2:k2){
          x2 = cbind(x2,y[cc2-j]);
       }
      }
      a2     = coef(lm(y2~x2));
      re2    = sum(resid(lm(y2~x2))^2);
      resid2 = resid(lm(y2~x2));
      aicm2[k2] = aic2=length(y2)*log(re2/length(y2))+2*(k2+1);
      if(aic2<minaic2){
        mr2     = resid2;
        minaic2 = aic2;
        minre2  = re2;
        ma2     = a2;
        mcc2    = cc2;
      }
  }
}    # end loop if(length(c2)>=3)

if(length(c2)<3) { mr2=minre2=mcc2=ma2=mink2=minaic2=0;}

if(length(c34)>=3){
  for(k34 in 1:kmax){
    cc34 = c34[which(c34>k34)];
    y34  = y[cc34];
    x34  = y[cc34-1];
    if(k34>1){
      for(j in 2:k34){
        x34 = cbind(x34,y[cc34-j]);
      }
    }
    a34     = coef(lm(y34~x34));
    re34    = sum(resid(lm(y34~x34))^2);
    resid34 = resid(lm(y34~x34));
    aic34   = aicm34[k34]=length(y34)*log(re34/length(y34))+2*(k34+1);
    if(aic34<minaic34){
      mr34     = resid34;
      minaic34 = aic34;
      minre34  = re34;
      ma34     = a34;
      mcc34    = cc34;
    }
  }
}    # end loop if(length(c34)>=3)

if(length(c34)<3) {mr34=minre34=mcc34=mink34=ma34=minaic34=0;}
  aic = minaic1+minaic2+minaic34;
  rss = minre1+minre2+minre34;
  result <- list(cc1=mcc1,cc2=mcc2,cc34=mcc34,a1=ma1,a2=ma2,a34=ma34,re1=mr1,re2=mr2,re34=mr34,rss=rss,aic=aic);
  return(result);
}

if(sum(regime==c(1,2,3,1))==4){
  minaic14 = minaic2=minaic3=999999;
  kmax     = 10;
  c14      = c(c1,c4);
  aicm14   = aicm2=aicm3=rep(0,10);
  if(length(c14)>=3){
     for(k14 in 1:kmax){
        cc14 = c14[which(c14>k14)];
        y14 = y[cc14];
        x14 = y[cc14-1];
        if(k14>1){
          for(j in 2:k14){
            x14 = cbind(x14,y[cc14-j])
           }
         }
         a14     = coef(lm(y14~x14));
         re14    = sum(resid(lm(y14~x14))^2);
         resid14 = resid(lm(y14~x14));
         aicm14[k14] = aic14=length(y14)*log(re14/length(y14))+2*(k14+1);
         if(aic14<minaic14){
           minaic14 = aic14;
           minre14  = re14;
           mr14     = resid14;
           ma14     = a14;
           mcc14    = cc14;
         }
     } 
}       # end loop if(sum(regime==c(1,2,3,1))==4)

if(length(c14)<3){mr14=minre14=mcc14=ma14=mink14=minaic14=0;}                  

if(length(c2)>=3){
  for(k2 in 1:kmax){ 
    cc2=c2[which(c2>k2)];    
    y2=y[cc2];
    x2=y[cc2-1];
    if(k2>1){
       for(j in 2:k2){
          x2=cbind(x2,y[cc2-j]);
        }
     }
     a2=coef(lm(y2~x2));
     re2=sum(resid(lm(y2~x2))^2);
     resid2=resid(lm(y2~x2));
     aicm2[k2]=aic2=length(y2)*log(re2/length(y2))+2*(k2+1);
     if(aic2<minaic2){
        mr2=resid2;
        minaic2=aic2;
        minre2=re2;
        ma2=a2;
        mcc2=cc2;
     }
   }
}

if(length(c2)<3) { mr2=minre2=mcc2=ma2=mink2=minaic2=0;}

if(length(c3)>=3){
  for(k3 in 1:kmax){
    cc3=c3[which(c3>k3)];
    y3=y[cc3];
    x3=y[cc3-1];
    if(k3>1){
      for(j in 2:k3){
        x3=cbind(x3,y[cc3-j]);
      }
    }
    a3=coef(lm(y3~x3));
    re3=sum(resid(lm(y3~x3))^2);
    resid3=resid(lm(y3~x3));
    aicm3[k3]=aic3=length(y3)*log(re3/length(y3))+2*(k3+1);
    if(aic3<minaic3){
      mr3=resid3;
      minaic3=aic3;
      minre3=re3;
      ma3=a3;
      mcc3=cc3;
    }
  }
}

if(length(c3)<3) { mr3=minre3=mcc3=mink3=ma3=minaic3=0;}
  aic=minaic14+minaic2+minaic3;
  rss=minre14+minre2+minre3;
  result<-list(cc14=mcc14,cc2=mcc2,cc3=mcc3,a14=ma14,a2=ma2,a3=ma3,re14=mr14,re2=mr2,re3=mr3,rss=rss,aic=aic)
  return(result);
}

if(sum(regime==c(1,2,1,3))==4){
  minaic13=minaic2=minaic4=999999;
  kmax=10;
  c13=c(c1,c3);
  aicm13=aicm2=aicm4=rep(0,10);
  if(length(c13)>=3){
    for(k13 in 1:kmax){
      cc13=c13[which(c13>k13)];
      y13=y[cc13];
      x13=y[cc13-1];
      if(k13>1){
        for(j in 2:k13){
          x13=cbind(x13,y[cc13-j])
        }
      }
      a13=coef(lm(y13~x13));
      re13=sum(resid(lm(y13~x13))^2);
      resid13=resid(lm(y13~x13));
      aicm13[k13]=aic13=length(y13)*log(re13/length(y13))+2*(k13+1);
      if(aic13<minaic13){
        minaic13=aic13;
        minre13=re13;
        mr13=resid13;
        ma13=a13;
        mcc13=cc13;
      }
   } 
}

if(length(c13)<3){mr13=minre13=mcc13=ma13=mink13=minaic13=0;}                  

if(length(c2)>=3){
  for(k2 in 1:kmax){ 
    cc2=c2[which(c2>k2)];    
    y2=y[cc2];
    x2=y[cc2-1];
    if(k2>1){
      for(j in 2:k2){
        x2=cbind(x2,y[cc2-j]);
      }
    }
    a2=coef(lm(y2~x2));
    re2=sum(resid(lm(y2~x2))^2);
    resid2=resid(lm(y2~x2));
    aicm2[k2]=aic2=length(y2)*log(re2/length(y2))+2*(k2+1);
    if(aic2<minaic2){
      mr2=resid2;
      minaic2=aic2;
      minre2=re2;
      ma2=a2;
      mcc2=cc2;
    }
  }
}

if(length(c2)<3) { mr2=minre2=mcc2=ma2=mink2=minaic2=0;}

if(length(c4)>=3){
  for(k4 in 1:kmax){
    cc4=c4[which(c4>k4)];
    y4=y[cc4];
    x4=y[cc4-1];
    if(k4>1){
      for(j in 2:k4){
        x4=cbind(x4,y[cc4-j])
      }
    }
    a4=coef(lm(y4~x4));
    resid4=resid(lm(y4~x4));
    re4=sum(resid(lm(y4~x4))^2); 
    aic4=aicm4[k4]=length(y4)*log(re4/length(y4))+2*(k4+1);
    if(aic4<minaic4){
      minaic4=aic4;
      mr4=resid4;
      minre4=re4;
      ma4=a4;
      mcc4=cc4;
    }
  }
}

if(length(c4)<3){ mr4=minre4=mink4=mcc4=ma4=minaic4=0;}
  aic=minaic13+minaic2+minaic4;
  rss=minre13+minre2+minre4;
  result<-list(cc13=mcc13,cc2=mcc2,cc4=mcc4,a13=ma13,a2=ma2,a4=ma4,re13=mr13,re2=mr2,re4=mr4,rss=rss,aic=aic)
  return(result);
}

if(sum(regime==c(1,2,3,2))==4){
  minaic24=minaic3=minaic1=999999;
  kmax=10;
  c24=c(c2,c4);
  aicm24=aicm1=aicm3=rep(0,10);
  if(length(c24)>=3){
    for(k24 in 1:kmax){
      cc24=c24[which(c24>k24)];
      y24=y[cc24];
      x24=y[cc24-1];
      if(k24>1){
        for(j in 2:k24){
          x24=cbind(x24,y[cc24-j])
        }
      }
      a24=coef(lm(y24~x24));
      re24=sum(resid(lm(y24~x24))^2);
      resid24=resid(lm(y24~x24));
      aicm24[k24]=aic24=length(y24)*log(re24/length(y24))+2*(k24+1);
      if(aic24<minaic24){
        minaic24=aic24;
        minre24=re24;
        mr24=resid24;
        ma24=a24;
        mcc24=cc24;
      }
   } 
}

if(length(c24)<3){mr24=minre24=mcc24=ma24=mink24=minaic24=0;}                  

if(length(c3)>=3){
  for(k3 in 1:kmax){
    cc3=c3[which(c3>k3)];
    y3=y[cc3];
    x3=y[cc3-1];
    if(k3>1){
      for(j in 2:k3){
        x3=cbind(x3,y[cc3-j]);
      }
    }
    a3=coef(lm(y3~x3));
    re3=sum(resid(lm(y3~x3))^2);
    resid3=resid(lm(y3~x3));
    aicm3[k3]=aic3=length(y3)*log(re3/length(y3))+2*(k3+1);
    if(aic3<minaic3){
      mr3=resid3;
      minaic3=aic3;
      minre3=re3;
      ma3=a3;
      mcc3=cc3;
    }
  }
}

if(length(c3)<3) { mr3=minre3=mcc3=mink3=ma3=minaic3=0;}

if(length(c1)>=3){
  for(k1 in 1:kmax){
    cc1=c1[which(c1>k1)];
    y1=y[cc1];
    x1=y[cc1-1];
    if(k1>1){
      for(j in 2:k1){
        x1=cbind(x1,y[cc1-j])
      }
    }
    a1=coef(lm(y1~x1));
    re1=sum(resid(lm(y1~x1))^2);
    resid1=resid(lm(y1~x1));
    aicm1[k1]=aic1=length(y1)*log(re1/length(y1))+2*(k1+1);
    if(aic1<minaic1){
       minaic1=aic1;
       minre1=re1;
       mr1=resid1;
       ma1=a1;
       mcc1=cc1;
     }
   } 
}

if(length(c1)<3){mr1=minre1=ma1=mcc1=mink1=minaic1=0;
            }                  
  aic=minaic24+minaic1+minaic3;
  rss=minre24+minre1+minre3;
  result<-list(cc24=mcc24,cc1=mcc1,cc3=mcc3,a24=ma24,a1=ma1,a3=ma3,re24=mr24,re1=mr1,re3=mr3,rss=rss,aic=aic)
  return(result);
}

if(sum(regime==c(1,1,2,2))==4){
   minaic12=minaic34=999999;
   aicm12=aicm34=rep(0,10);
   c12=c(c1,c2);
   c34=c(c3,c4);
   if(length(c12)>=3){
     for(k12 in 1:kmax){
       cc12=c12[which(c12>k12)];
       y12=y[cc12];
       x12=y[cc12-1];
       if(k12>1){
         for(j in 2:k12){
           x12=cbind(x12,y[cc12-j])
         }
       }
       a12=coef(lm(y12~x12));
       re12=sum(resid(lm(y12~x12))^2);
       resid12=resid(lm(y12~x12));
       aicm12[k12]=aic12=length(y12)*log(re12/length(y12))+2*(k12+1);
       if(aic12<minaic12){
          minaic12=aic12;
          minre12=re12;
          mr12=resid12;
          ma12=a12;
          mcc12=cc12;
        }
    } 
}

if(length(c12)<3){ minre12=mcc12=ma12=mink12=mr12=minaic12=0;  }                  

if(length(c34)>=3){
  for(k34 in 1:kmax){
    cc34=c34[which(c34>k34)];
    y34=y[cc34];
    x34=y[cc34-1];
    if(k34>1){
      for(j in 2:k34){
        x34=cbind(x34,y[cc34-j]);
      }
    }
    a34=coef(lm(y34~x34));
    re34=sum(resid(lm(y34~x34))^2);
    resid34=resid(lm(y34~x34));
    aic34=aicm34[k34]=length(y34)*log(re34/length(y34))+2*(k34+1);
    if(aic34<minaic34){
       mr34=resid34;
       minaic34=aic34;
       minre34=re34;
       ma34=a34;
       mcc34=cc34;
     }
   } 
}

if(length(c34)<3) { mr34=minre34=mcc34=mink34=ma34=minaic34=0;}
  aic=minaic12+minaic34;
  rss=minre12+minre34;
  result<-list(cc12=mcc12,cc34=mcc34,a12=ma12,a34=ma34,re12=mr12,re34=mr34,rss=rss,aic=aic)
  return(result);
}

if(sum(regime==c(1,2,1,2))==4){
   minaic13=minaic24=999999;
   aicm13=aicm24=rep(0,10);
   c13=c(c1,c3);
   c24=c(c2,c4);
   if(length(c24)>=3){
     for(k24 in 1:kmax){
       cc24=c24[which(c24>k24)];
       y24=y[cc24];
       x24=y[cc24-1];
       if(k24>1){
         for(j in 2:k24){
           x24=cbind(x24,y[cc24-j])
         }
       }
       a24=coef(lm(y24~x24));
       re24=sum(resid(lm(y24~x24))^2);
       resid24=resid(lm(y24~x24));
       aicm24[k24]=aic24=length(y24)*log(re24/length(y24))+2*(k24+1);
       if(aic24<minaic24){
         minaic24=aic24;
         minre24=re24;
         mr24=resid24;
         ma24=a24;
         mcc24=cc24;
       }
    } 
}

if(length(c24)<3){mr24=minre24=mcc24=ma24=mink24=minaic24=0;}   
   aicm13=aicm2=aicm4=rep(0,10);
   if(length(c13)>=3){
     for(k13 in 1:kmax){
       cc13=c13[which(c13>k13)];
       y13=y[cc13];
       x13=y[cc13-1];
       if(k13>1){
         for(j in 2:k13){
           x13=cbind(x13,y[cc13-j])
         }
       }
       a13=coef(lm(y13~x13));
       re13=sum(resid(lm(y13~x13))^2);
       resid13=resid(lm(y13~x13));
       aicm13[k13]=aic13=length(y13)*log(re13/length(y13))+2*(k13+1);
       if(aic13<minaic13){
          minaic13=aic13;
          minre13=re13;
          mr13=resid13;
          ma13=a13;
          mcc13=cc13;
        }
      } 
}

if(length(c13)<3){mr13=minre13=mcc13=ma13=mink13=minaic13=0;}              
  aic=minaic13+minaic24;
  rss=minre13+minre24;
  result<-list(cc13=mcc13,cc24=mcc24,a13=ma13,a24=ma24,re13=mr13,re24=mr24,rss=rss,aic=aic)
  return(result);
}

if(sum(regime==c(1,2,2,1))==4){
  minaic14=minaic23=999999;
  aicm14=aicm23=rep(0,10);
  c14=c(c1,c4);
  c23=c(c2,c3);
  if(length(c23)>=3){
    for(k23 in 1:kmax){ 
      cc23=c23[which(c23>k23)];    
      y23=y[cc23];
      x23=y[cc23-1];
      if(k23>1){
        for(j in 2:k23){
          x23=cbind(x23,y[cc23-j]);
        }
      }
      a23=coef(lm(y23~x23));
      re23=sum(resid(lm(y23~x23))^2);
      resid23=resid(lm(y23~x23));
      aicm23[k23]=aic23=length(y23)*log(re23/length(y23))+2*(k23+1);
      if(aic23<minaic23){
        mr23=resid23;
        minaic23=aic23;
        minre23=re23;
        ma23=a23;
        mcc23=cc23;
      }
  }
}

if(length(c23)<3) { mr23=minre23=mcc23=ma23=mink23=minaic23=0;}

if(length(c14)>=3){
  for(k14 in 1:kmax){
    cc14=c14[which(c14>k14)];
    y14=y[cc14];
    x14=y[cc14-1];
    if(k14>1){
      for(j in 2:k14){
        x14=cbind(x14,y[cc14-j])
      }
    }
    a14=coef(lm(y14~x14));
    re14=sum(resid(lm(y14~x14))^2);
    resid14=resid(lm(y14~x14));
    aicm14[k14]=aic14=length(y14)*log(re14/length(y14))+2*(k14+1);
    if(aic14<minaic14){
       minaic14=aic14;
       minre14=re14;
       mr14=resid14;
       ma14=a14;
       mcc14=cc14;
     }
   } 
}

if(length(c14)<3){mr14=minre14=mcc14=ma14=mink14=minaic14=0;}                  
  aic=minaic14+minaic23;
  rss=minre14+minre23;
  result<-list(cc14=mcc14,cc23=mcc23,a14=ma14,a23=ma23,re14=mr14,re23=mr23,rss=rss,aic=aic)
  return(result);
}


if(sum(regime==c(1,1,1,2))==4){
   minaic123=minaic4=999999;
   aicm123=aicm4=rep(0,10);

   c123=c(c1,c2,c3);
   if(length(c123)>=3){
     for(k123 in 1:kmax){ 
       cc123=c123[which(c123>k123)];    
       y123=y[cc123];
       x123=y[cc123-1];
       if(k123>1){
         for(j in 2:k123){
           x123=cbind(x123,y[cc123-j]);
         }
       }
       a123=coef(lm(y123~x123));
       re123=sum(resid(lm(y123~x123))^2);
       resid123=resid(lm(y123~x123));
       aicm123[k123]=aic123=length(y123)*log(re123/length(y123))+2*(k123+1);
       if(aic123<minaic123){
         mr123=resid123;
         minaic123=aic123;
         minre123=re123;
         ma123=a123;
         mcc123=cc123;
       }
    }
}

if(length(c123)<3) {mr123=minre123=mcc123=ma123=mink123=minaic123=0;}

if(length(c4)>=3){
  for(k4 in 1:kmax){
    cc4=c4[which(c4>k4)];
    y4=y[cc4];
    x4=y[cc4-1];
    if(k4>1){
      for(j in 2:k4){
        x4=cbind(x4,y[cc4-j])
      }
    }
    a4=coef(lm(y4~x4));
    re4=sum(resid(lm(y4~x4))^2);
    resid4=resid(lm(y4~x4));
    aicm4[k4]=aic4=length(y4)*log(re4/length(y4))+2*(k4+1);
    if(aic4<minaic4){
      minaic4=aic4;
      minre4=re4;
      mr4=resid4;
      ma4=a4;
      mcc4=cc4;
    }
  } 
}

if(length(c4)<3){mr4=minre4=mcc4=ma4=mink4=minaic4=0;}                  

  aic=minaic4+minaic123;
  rss=minre4+minre123;
  result<-list(cc4=mcc4,cc123=mcc123,a4=ma4,a123=ma123,re4=mr4,re123=mr123,rss=rss,aic=aic)
  return(result);
}

if(sum(regime==c(1,1,2,1))==4){
  minaic124=minaic3=999999;
  aicm124=aicm3=rep(0,10);

  c124=c(c1,c2,c4);
  if(length(c124)>=3){
    for(k124 in 1:kmax){ 
      cc124=c124[which(c124>k124)];    
      y124=y[cc124];
      x124=y[cc124-1];
      if(k124>1){
        for(j in 2:k124){
          x124=cbind(x124,y[cc124-j]);
        }
       }
       a124=coef(lm(y124~x124));
       re124=sum(resid(lm(y124~x124))^2);
       resid124=resid(lm(y124~x124));
       aicm124[k124]=aic124=length(y124)*log(re124/length(y124))+2*(k124+1);
       if(aic124<minaic124){
         mr124=resid124;
         minaic124=aic124;
         minre124=re124;
         ma124=a124;
         mcc124=cc124;
       }
   }
}

if(length(c124)<3) {mr124=minre124=mcc124=ma124=mink124=minaic124=0;}

if(length(c3)>=3){
  for(k3 in 1:kmax){
    cc3=c3[which(c3>k3)];
    y3=y[cc3];
    x3=y[cc3-1];
    if(k3>1){
      for(j in 2:k3){
        x3=cbind(x3,y[cc3-j])
      }
    }
    a3=coef(lm(y3~x3));
    re3=sum(resid(lm(y3~x3))^2);
    resid3=resid(lm(y3~x3));
    aicm3[k3]=aic3=length(y3)*log(re3/length(y3))+2*(k3+1);
    if(aic3<minaic3){
      minaic3=aic3;
      minre3=re3;
      mr3=resid3;
      ma3=a3;
      mcc3=cc3;
    }
 } 
}

if(length(c3)<3){mr3=minre3=mcc3=ma3=mink3=minaic3=0;}                  

  aic=minaic3+minaic124;
  rss=minre3+minre124;
  result<-list(cc3=mcc3,cc124=mcc124,a3=ma3,a124=ma124,re3=mr3,re124=mr124,rss=rss,aic=aic)
  return(result);
}


if(sum(regime==c(1,1,1,2))==4){
   minaic123=minaic4=999999;
   aicm123=aicm4=rep(0,10);

   c123=c(c1,c2,c3);
   if(length(c123)>=3){
     for(k123 in 1:kmax){ 
       cc123=c123[which(c123>k123)];    
       y123=y[cc123];
       x123=y[cc123-1];
       if(k123>1){
         for(j in 2:k123){
           x123=cbind(x123,y[cc123-j]);
         }
       }
       a123=coef(lm(y123~x123));
       re123=sum(resid(lm(y123~x123))^2);
       resid123=resid(lm(y123~x123));
       aicm123[k123]=aic123=length(y123)*log(re123/length(y123))+2*(k123+1);
       if(aic123<minaic123){
         mr123=resid123;
         minaic123=aic123;
         minre123=re123;
         ma123=a123;
         mcc123=cc123;
       }
   }
}

if(length(c123)<3) { mr123=minre123=mcc123=ma123=mink123=minaic123=0;}

if(length(c4)>=3){
  for(k4 in 1:kmax){
    cc4=c4[which(c4>k4)];
    y4=y[cc4];
    x4=y[cc4-1];
    if(k4>1){
      for(j in 2:k4){
        x4=cbind(x4,y[cc4-j])
      }
    }
    a4=coef(lm(y4~x4));
    re4=sum(resid(lm(y4~x4))^2);
    resid4=resid(lm(y4~x4));
    aicm4[k4]=aic4=length(y4)*log(re4/length(y4))+2*(k4+1);
    if(aic4<minaic4){
      minaic4=aic4;
      minre4=re4;
      mr4=resid4;
      ma4=a4;
      mcc4=cc4;
    }
  } 
}

if(length(c4)<3){mr4=minre4=mcc4=ma4=mink4=minaic4=0;}                  

  aic=minaic4+minaic123;
  rss=minre4+minre123;
  result<-list(cc4=mcc4,cc123=mcc123,a4=ma4,a123=ma123,re4=mr4,re123=mr123,rss=rss,aic=aic)
  return(result);
}

if(sum(regime==c(1,2,1,1))==4){
  minaic134=minaic2=999999;
  aicm134=aicm2=rep(0,10);

  c134=c(c1,c3,c4);
  if(length(c134)>=3){
    for(k134 in 1:kmax){ 
      cc134=c134[which(c134>k134)];    
      y134=y[cc134];
      x134=y[cc134-1];
      if(k134>1){
        for(j in 2:k134){
          x134=cbind(x134,y[cc134-j]);
        }
      }
      a134=coef(lm(y134~x134));
      re134=sum(resid(lm(y134~x134))^2);
      resid134=resid(lm(y134~x134));
      aicm134[k134]=aic134=length(y134)*log(re134/length(y134))+2*(k134+1);
      if(aic134<minaic134){
        mr134=resid134;
        minaic134=aic134;
        minre134=re134;
        ma134=a134;
        mcc134=cc134;
      }
   }
}

if(length(c134)<3) { mr134=minre134=mcc134=ma134=mink134=minaic134=0;}

if(length(c2)>=3){
  for(k2 in 1:kmax){
    cc2=c2[which(c2>k2)];
    y2=y[cc2];
    x2=y[cc2-1];
    if(k2>1){
      for(j in 2:k2){
        x2=cbind(x2,y[cc2-j])
      }
    }
    a2=coef(lm(y2~x2));
    re2=sum(resid(lm(y2~x2))^2);
    resid2=resid(lm(y2~x2));
    aicm2[k2]=aic2=length(y2)*log(re2/length(y2))+2*(k2+1);
    if(aic2<minaic2){
      minaic2=aic2;
      minre2=re2;
      mr2=resid2;
      ma2=a2;
      mcc2=cc2;
    }
  } 
}

if(length(c2)<2){minresid2=minre2=mincc2=mina2=mink2=minaic2=0;}                  

  aic=minaic2+minaic134;
  rss=minre2+minre134;
  result<-list(cc2=mcc2,cc134=mcc134,a2=ma2,a134=ma134,re2=mr2,re134=mr134,rss=rss,aic=aic)
  return(result);
}


if(sum(regime==c(1,2,2,2))==4){
  minaic234=minaic1=999999;
  aicm234=aicm1=rep(0,10);

  c234=c(c2,c3,c4);
  if(length(c234)>=3){
    for(k234 in 1:kmax){ 
      cc234=c234[which(c234>k234)];    
      y234=y[cc234];
      x234=y[cc234-1];
      if(k234>1){
        for(j in 2:k234){
          x234=cbind(x234,y[cc234-j]);
        }
      }
      a234=coef(lm(y234~x234));
      re234=sum(resid(lm(y234~x234))^2);
      resid234=resid(lm(y234~x234));
      aicm234[k234]=aic234=length(y234)*log(re234/length(y234))+2*(k234+1);
      if(aic234<minaic234){
        mr234=resid234;
        minaic234=aic234;
        minre234=re234;
        ma234=a234;
        mcc234=cc234;
      }
   }
}

if(length(c234)<3) { mr234=minre234=mcc234=ma234=mink234=minaic234=0;}

if(length(c1)>=3){
  for(k1 in 1:kmax){
    cc1=c1[which(c1>k1)];
    y1=y[cc1];
    x1=y[cc1-1];
    if(k1>1){
      for(j in 2:k1){
        x1=cbind(x1,y[cc1-j])
      }
    }
    a1=coef(lm(y1~x1));
    re1=sum(resid(lm(y1~x1))^2);
    resid1=resid(lm(y1~x1));
    aicm1[k1]=aic1=length(y1)*log(re1/length(y1))+2*(k1+1);
    if(aic1<minaic1){
      minaic1=aic1;
      minre1=re1;
      mr1=resid1;
      ma1=a1;
      mcc1=cc1;
    }
  } 
}

if(length(c1)<3){     minre1=ma1=mcc1=mink1=mr1=minaic1=0;}                  
  aic=minaic1+minaic234;
  rss=minre1+minre234;
  result<-list(cc1=mcc1,cc234=mcc234,a1=ma1,a234=ma234,re1=mr1,re234=mr234,rss=rss,aic=aic)
  return(result);
}

if(sum(regime==c(1,1,1,1))==4){
   minaic1234=999999;
   
   aicm1234=rep(0,10);
   c1234=c(c1,c2,c3,c4);
   if(length(c1234)>=3){
     for(k1234 in 1:kmax){ 
       cc1234=c1234[which(c1234>k1234)];    
       y1234=y[cc1234];
       x1234=y[cc1234-1];
       if(k1234>1){
         for(j in 2:k1234){
           x1234=cbind(x1234,y[cc1234-j]);
         }
       }
       a1234=coef(lm(y1234~x1234));
       re1234=sum(resid(lm(y1234~x1234))^2);
       resid1234=resid(lm(y1234~x1234));
       aicm1234[k1234]=aic1234=length(y1234)*log(re1234/length(y1234))+2*(k1234+1);
       if(aic1234<minaic1234){
         minresid1234=resid1234;
         minaic1234=aic1234;
         minre1234=re1234;
         mina1234=a1234;
         mincc1234=cc1234;
       }
    }
}

if(length(c1234)<3) {minresid1234=minre1234=mincc1234=mina1234=mink1234=minaic1234=0;}
  aic=minaic1234;
  re=minre1234;
  result<-list(cc1234=mincc1234,a1234=mina1234,re1234=minresid1234,rss=re,aic=aic)
  return(result);
}

}  # end function

