###############################################################################################
# R code
# File: threshold_est_with_intercept.r
#
# This program is designed by H. Chen and T.L. Chong(2005) to estimate the M-TAR model with two
# threshold variables. 
#
# y      = the time series data.  
# Z1, Z2 = two vectors as the threshold variables.
#
# Searching area is between the (lowb,upb)=(0.15,0.85) rank of the sample.
# order is the vector indicating the AR order in each regime. For example,
# c(1,1,1,1) means each regime has AR(1) process.
#
# regime is a 4 by 1 vector, which indicates the model struture in different regimes. 
# For detail, c(1,2,3,4) means that the model is four-regime model; 
#             c(1,1,2,3) means that the model is three-regime model and the regime I 
#                        and regime II should be combined together.  
# Similarly,  c(1,2,1,3) means that the model is three-regime model and the regime I and 
#                        regime III should be combined together.
# Moreover,   c(1,1,1,2) means that the model is two-regime model and regime I, II, III 
#                        should be put together.
#
# grid = the number of grid points  for searching area. 
#
# The function returns the estimated coefficients of the model and the estimated 
# threshold values.
#
# Reference: 
# Chen, H., Chong, T.T.L., and Bai, J. (2012).
#    Theory and applications of TAR model with two threshold variables.
#    Econometric Reviews, 31(2), 142-170.
#    DOI: 10.1080/07474938.2011.607100.


################################################################################################
thresholdest<-function(y,z1,z2,regime,order){

grid = 100;
lowb = 0.15;
upb  = 0.85

lz=length(z1);
ly=length(y);

if(lz!=ly){
  print("the size of the sample does not match");
  return(0);  
}

if(max(regime)!=length(order))
  {
   print("the order does not match the regime indicator");
   return(0);
  }

lamda1=floor(0.9*length(z1));
lamda2=floor(0.1*length(z1));
lamda3=floor(0.9*length(z2));
lamda4=floor(0.1*length(z2));

a1=sort(z1)[lamda1]
a2=sort(z1)[lamda2]
a3=sort(z2)[lamda3]
a4=sort(z2)[lamda4]

rr1=seq(a2,a1,(a1-a2)/grid);
rr2=seq(a4,a3,(a3-a4)/grid);
t2=length(rr1);
t4=length(rr2);
t1=1;
t3=1;

maic=minre=99999;

if(sum(regime==c(1,2,3,4))==4){
  for(r1 in t1:t2){
    for(r2 in t3:t4){
      c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
      c2=which(z1<rr1[r1]&z2>=rr2[r2]);
      c3=which(z1<rr1[r1]&z2<rr2[r2]);
      c4=which(z1>=rr1[r1]&z2<rr2[r2]);
      if(length(c1)>=3){
        k1=order[1];
        cc1=c1[which(c1>k1)];
        y1=y[cc1];
        x1=y[cc1-1];
        if(k1>1){
          for(j in 2:k1){
            x1=cbind(x1,y[cc1-j])
          }
        }
        a1=coef(lm(y1~x1));
        re1=sum(resid(lm(y1~x1))^2);
        resid1=resid(lm(y1~x1));
        aic1=length(y1)*log(re1/length(y1))+2*(k1+1);
      }
      
      if(length(c1)<3){ 
        re1=a1=aic1=0;
      } 
      
      if(length(c2)>=3){
        k2=order[2];
        cc2=c2[which(c2>k2)];    
        y2=y[cc2];
        x2=y[cc2-1];
        if(k2>1){
          for(j in 2:k2){
            x2=cbind(x2,y[cc2-j]);
          }
        }
        a2=coef(lm(y2~x2));
        re2=sum(resid(lm(y2~x2))^2);
        resid2=resid(lm(y2~x2));
        aic2=length(y2)*log(re2/length(y2))+2*(k2+1);
      } 
      
      if(length(c2)<3) { re2=a2=k2=aic2=0;}
        if(length(c3)>=3){
          k3=order[3];
          cc3=c3[which(c3>k3)];
          y3=y[cc3];
          x3=y[cc3-1];
          if(k3>1){
            for(j in 2:k3){
              x3=cbind(x3,y[cc3-j]);
            }
          }
          a3=coef(lm(y3~x3));
          re3=sum(resid(lm(y3~x3))^2);
          resid3=resid(lm(y3~x3));
          aic3=length(y3)*log(re3/length(y3))+2*(k3+1);
        }

        if(length(c3)<3) { re3=k3=a3=aic3=0;}
          if(length(c4)>=3){
            k4=order[4];
            cc4=c4[which(c4>k4)];
            y4=y[cc4];
            x4=y[cc4-1];
            if(k4>1){
              for(j in 2:k4){
                x4=cbind(x4,y[cc4-j])
              }
            }
            a4=coef(lm(y4~x4));
            resid4=resid(lm(y4~x4));
            re4=sum(resid(lm(y4~x4))^2); 
            aic4=length(y4)*log(re4/length(y4))+2*(k4+1);
          }
          
          if(length(c4)<3){ re4=k4=a4=aic4=0;}
            re=(re1+re2+re3+re4);
            maic=(aic1+aic2+aic3+aic4);
            if(re<minre){
              r1min=rr1[r1];
              r2min=rr2[r2];
              minre=re;
              mina1=a1;
              mina2=a2;
              mina3=a3;
              mina4=a4;
              aic=maic;
             }                            
   } #  end loop for(r2 in t3:t4)
 }   #  end loop for(r1 in t1:t2)
 threshold=c(r1min,r2min);
 result<-list(a1=mina1,a2=mina2,a3=mina3,a4=mina4,threshold=threshold,rss=minre,aic=aic);
 return(result);
}    # end loop if(sum(regime==c(1,2,3,4))==4)


if(sum(regime==c(1,1,2,3))==4){
for(r1 in t1:t2){
  for(r2 in t3:t4){
    c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
    c2=which(z1<rr1[r1]&z2>=rr2[r2]);
    c3=which(z1<rr1[r1]&z2<rr2[r2]);
    c4=which(z1>=rr1[r1]&z2<rr2[r2]);
    c12=c(c1,c2);
    if(length(c12)>=3){
      k12=order[1];
      cc12=c1[which(c12>k12)];
      y12=y[cc12];
      x12=y[cc12-1];
      if(k12>1){
        for(j in 2:k12){
          x12=cbind(x12,y[cc12-j])
        }
      }
      a12=coef(lm(y12~x12));
      re12=sum(resid(lm(y12~x12))^2);
      resid12=resid(lm(y12~x12));
      aic12=length(y12)*log(re12/length(y12))+2*(k12+1);
    }
    
    if(length(c12)<3){ 
      re12=a12=aic12=0;
    }                  
                      
    if(length(c3)>=3){
      k3=order[2];
      cc3=c3[which(c3>k3)];
      y3=y[cc3];
      x3=y[cc3-1];
      if(k3>1){
        for(j in 2:k3){
          x3=cbind(x3,y[cc3-j]);
        }
      }
      a3=coef(lm(y3~x3));
      re3=sum(resid(lm(y3~x3))^2);
      resid3=resid(lm(y3~x3));
      aic3=length(y3)*log(re3/length(y3))+2*(k3+1);
    }
    
    if(length(c3)<3) { re3=k3=a3=aic3=0;}
      if(length(c4)>=3){
        k4=order[3];
        cc4=c4[which(c4>k4)];
        y4=y[cc4];
        x4=y[cc4-1];
        if(k4>1){
          for(j in 2:k4){
            x4=cbind(x4,y[cc4-j])
          }
        }
        a4=coef(lm(y4~x4));
        resid4=resid(lm(y4~x4));
        re4=sum(resid(lm(y4~x4))^2); 
        aic4=length(y4)*log(re4/length(y4))+2*(k4+1);
                                          }
        if(length(c4)<3){ re4=k4=a4=aic4=0;}

        re=(re12+re3+re4);
        maic=(aic12+aic3+aic4);
        if(re<minre){
            r1min=rr1[r1];
            r2min=rr2[r2];
            minre=re;
            mina12=a12;
            mina3=a3;
            mina4=a4;
            aic=maic;
         }                           
     } # end loop for(r2 in t3:t4)
  }    # end loop for(r1 in t1:t2)
  threshold=c(r1min,r2min);
  result<-list(a12=mina12,a3=mina3,a4=mina4,threshold=threshold,rss=minre,aic=aic);
  return(result);
}  # end loop if(sum(regime==c(1,1,2,3))==4)

if(sum(regime==c(1,2,1,3))==4){
for(r1 in t1:t2){
  for(r2 in t3:t4){
    c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
    c2=which(z1<rr1[r1]&z2>=rr2[r2]);
    c3=which(z1<rr1[r1]&z2<rr2[r2]);
    c4=which(z1>=rr1[r1]&z2<rr2[r2]);
    c13=c(c1,c3);
    if(length(c13)>=3){
      k13=order[1];
      cc13=c13[which(c13>k13)];
      y13=y[cc13];
      x13=y[cc13-1];
      if(k13>1){
        for(j in 2:k13){
          x13=cbind(x13,y[cc13-j])
        }
      }
      a13=coef(lm(y13~x13));
      re13=sum(resid(lm(y13~x13))^2);
      resid13=resid(lm(y13~x13));
      aic13=length(y13)*log(re13/length(y13))+2*(k13+1);
                                         }
      if(length(c13)<3){ 
        re13=a13=aic13=0;
      }                  
      
       if(length(c2)>=3){
         k2=order[2];
         cc2=c2[which(c2>k2)];    
         y2=y[cc2];
         x2=y[cc2-1];
         if(k2>1){
           for(j in 2:k2){
             x2=cbind(x2,y[cc2-j]);
           }
         }
       
         a2=coef(lm(y2~x2));
         re2=sum(resid(lm(y2~x2))^2);
         resid2=resid(lm(y2~x2));
         aic2=length(y2)*log(re2/length(y2))+2*(k2+1);
       }
       
       if(length(c2)<3) { re2=a2=k2=aic2=0;}
         if(length(c4)>=3){
           k4=order[3];
           cc4=c4[which(c4>k4)];
           y4=y[cc4];
           x4=y[cc4-1];
           if(k4>1){
           for(j in 2:k4){
             x4=cbind(x4,y[cc4-j])
           }
         }
         
         a4=coef(lm(y4~x4));
         resid4=resid(lm(y4~x4));
         re4=sum(resid(lm(y4~x4))^2); 
         aic4=length(y4)*log(re4/length(y4))+2*(k4+1);
       }
       
       if(length(c4)<3){ re4=k4=a4=aic4=0;}
      
       re=(re13+re2+re4);
       maic=(aic13+aic2+aic4);
       if(re<minre){
         r1min=rr1[r1];
         r2min=rr2[r2];
         minre=re;
         mina13=a13;
         mina2=a2;
         mina4=a4;
         aic=maic;
       }   
    }
  }
  threshold=c(r1min,r2min);
  result<-list(a13=mina13,a2=mina2,a4=mina4,threshold=threshold,rss=minre,aic=aic)
  return(result);
} # end loop if(sum(regime==c(1,2,1,3))==4)

if(sum(regime==c(1,2,3,1))==4){
  for(r1 in t1:t2){
    for(r2 in t3:t4){
      c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
      c2=which(z1<rr1[r1]&z2>=rr2[r2]);
      c3=which(z1<rr1[r1]&z2<rr2[r2]);
      c4=which(z1>=rr1[r1]&z2<rr2[r2]);
      c14=c(c1,c4);
      if(length(c14)>=3){
        k14=order[1];
        cc14=c14[which(c14>k14)];
        y14=y[cc14];
        x14=y[cc14-1];
        if(k14>1){
          for(j in 2:k14){
            x14=cbind(x14,y[cc14-j])
          }
        }
        
        a14=coef(lm(y14~x14));
        re14=sum(resid(lm(y14~x14))^2);
        resid14=resid(lm(y14~x14));
        aic14=length(y14)*log(re14/length(y14))+2*(k14+1);
      }
      
      if(length(c14)<3){ 
        re14=a14=aic14=0;
      }                  
      
      if(length(c2)>=3){
        k2=order[2];
        cc2=c2[which(c2>k2)];    
        y2=y[cc2];
        x2=y[cc2-1];
        if(k2>1){
          for(j in 2:k2){
            x2=cbind(x2,y[cc2-j]);
          }
        }
        
        a2=coef(lm(y2~x2));
        re2=sum(resid(lm(y2~x2))^2);
        resid2=resid(lm(y2~x2));
        aic2=length(y2)*log(re2/length(y2))+2*(k2+1);
      }
      
      if(length(c2)<3) { re2=a2=k2=aic2=0;}
        if(length(c3)>=3){
          k3=order[3];
          cc3=c3[which(c3>k3)];
          y3=y[cc3];
          x3=y[cc3-1];
          if(k3>1){
            for(j in 2:k3){
              x3=cbind(x3,y[cc3-j]);
            }
          }
          
          a3=coef(lm(y3~x3));
          re3=sum(resid(lm(y3~x3))^2);
          resid3=resid(lm(y3~x3));
          aic3=length(y3)*log(re3/length(y3))+2*(k3+1);
        }
        
        if(length(c3)<3) { re3=k3=a3=aic3=0;}
          re=(re14+re2+re3);
          maic=(aic14+aic2+aic3);
          if(re<minre){
            r1min=rr1[r1];
            r2min=rr2[r2];
            minre=re;
            mina14=a14;
            mina2=a2;
            mina3=a3;
            aic=maic;
          }                             
      }
  }
  threshold=c(r1min,r2min);
  result<-list(a14=mina14,a2=mina2,a3=mina3,threshold=threshold,rss=minre,aic=aic)
  return(result);
}


if(sum(regime==c(1,2,2,3))==4){
  for(r1 in t1:t2){
    for(r2 in t3:t4){
      c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
      c2=which(z1<rr1[r1]&z2>=rr2[r2]);
      c3=which(z1<rr1[r1]&z2<rr2[r2]);
      c4=which(z1>=rr1[r1]&z2<rr2[r2]);
      c23=c(c2,c3);
      if(length(c1)>=3){
        k1=order[1];
        cc1=c1[which(c1>k1)];
        y1=y[cc1];
        x1=y[cc1-1];
        if(k1>1){
          for(j in 2:k1){
            x1=cbind(x1,y[cc1-j])
          }
        }
        
        a1=coef(lm(y1~x1));
        re1=sum(resid(lm(y1~x1))^2);
        resid1=resid(lm(y1~x1));
        aic1=length(y1)*log(re1/length(y1))+2*(k1+1);
      }
      
      if(length(c1)<3){ 
        re1=a1=aic1=0;
      }                  
      
      if(length(c23)>=3){
         k23=order[2];
         cc23=c23[which(c23>k23)];    
         y23=y[cc23];
         x23=y[cc23-1];
         if(k23>1){
           for(j in 2:k23){
             x23=cbind(x23,y[cc23-j]);
           }
         }
         
         a23=coef(lm(y23~x23));
         re23=sum(resid(lm(y23~x23))^2);
         resid23=resid(lm(y23~x23));
         aic23=length(y23)*log(re23/length(y23))+2*(k23+1);
                                        }
         if(length(c23)<3) { re23=a23=k23=aic23=0;}
           if(length(c4)>=3){
             k4=order[3];
             cc4=c4[which(c4>k4)];
             y4=y[cc4];
             x4=y[cc4-1];
             if(k4>1){
               for(j in 2:k4){
                 x4=cbind(x4,y[cc4-j])
               }
             }
             
             a4=coef(lm(y4~x4));
             resid4=resid(lm(y4~x4));
             re4=sum(resid(lm(y4~x4))^2); 
             aic4=length(y4)*log(re4/length(y4))+2*(k4+1);
           }
           
           if(length(c4)<3){ re4=k4=a4=aic4=0;}
             re=(re1+re23+re4);
             maic=(aic1+aic23+aic4);
             if(re<minre){
               r1min=rr1[r1];
               r2min=rr2[r2];
               minre=re;
               mina1=a1;
               mina23=a23;
               mina4=a4;
               aic=maic;
             }                           
     }
  }
  threshold=c(r1min,r2min);
  result<-list(a1=mina1,a23=mina23,a4=mina4,threshold=threshold,rss=minre,aic=aic)
  return(result);
}

if(sum(regime==c(1,2,3,2))==4){
for(r1 in t1:t2){
  for(r2 in t3:t4){
    c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
    c2=which(z1<rr1[r1]&z2>=rr2[r2]);
    c3=which(z1<rr1[r1]&z2<rr2[r2]);
    c4=which(z1>=rr1[r1]&z2<rr2[r2]);
    c24=c(c2,c4);
    if(length(c1)>=3){
      k1=order[1];
      cc1=c1[which(c1>k1)];
      y1=y[cc1];
      x1=y[cc1-1];
      if(k1>1){
        for(j in 2:k1){
          x1=cbind(x1,y[cc1-j])
        }
      }
      
      a1=coef(lm(y1~x1));
      re1=sum(resid(lm(y1~x1))^2);
      resid1=resid(lm(y1~x1));
      aic1=length(y1)*log(re1/length(y1))+2*(k1+1);
    }
    
    if(length(c1)<3){ 
      re1=a1=aic1=0;
    }                  
    
    if(length(c24)>=3){
      k24=order[2];
      cc24=c2[which(c24>k24)];    
      y24=y[cc24];
      x24=y[cc24-1];
      if(k24>1){
        for(j in 2:k24){
          x24=cbind(x24,y[cc24-j]);
        }
      }
      
      a24=coef(lm(y24~x24));
      re24=sum(resid(lm(y24~x24))^2);
      resid24=resid(lm(y24~x24));
      aic24=length(y24)*log(re24/length(y24))+2*(k24+1);
    }
    
    if(length(c24)<3) { re24=a24=k24=aic24=0;}
    
    if(length(c3)>=3){
       k3=order[3];
       cc3=c3[which(c3>k3)];
       y3=y[cc3];
       x3=y[cc3-1];
       if(k3>1){
         for(j in 2:k3){
           x3=cbind(x3,y[cc3-j]);
         }
       }
        
       a3=coef(lm(y3~x3));
       re3=sum(resid(lm(y3~x3))^2);
       resid3=resid(lm(y3~x3));
       aic3=length(y3)*log(re3/length(y3))+2*(k3+1);
     }
     
     if(length(c3)<3) { re3=k3=a3=aic3=0;}
                    
     re=(re1+re24+re3);
     maic=(aic1+aic24+aic3);
     if(re<minre){
       r1min=rr1[r1];
       r2min=rr2[r2];
       minre=re;
       mina1=a1;
       mina24=a24;
       mina3=a3;
       aic=maic;
     }                             
    }
 }
 threshold=c(r1min,r2min);
 result<-list(a1=mina1,a24=mina24,a3=mina3,threshold=threshold,rss=minre,aic=aic)
 return(result);
}

if(sum(regime==c(1,2,3,3))==4){
  for(r1 in t1:t2){
    for(r2 in t3:t4){
      c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
      c2=which(z1<rr1[r1]&z2>=rr2[r2]);
      c3=which(z1<rr1[r1]&z2<rr2[r2]);
      c4=which(z1>=rr1[r1]&z2<rr2[r2]);
      c34=c(c3,c4);
      if(length(c1)>=3){
        k1=order[1];
        cc1=c1[which(c1>k1)];
        y1=y[cc1];
        x1=y[cc1-1];
        if(k1>1){
          for(j in 2:k1){
            x1=cbind(x1,y[cc1-j])
          }
        }
        
        a1=coef(lm(y1~x1));
        re1=sum(resid(lm(y1~x1))^2);
        resid1=resid(lm(y1~x1));
        aic1=length(y1)*log(re1/length(y1))+2*(k1+1);
      }
      
      if(length(c1)<3){ 
        re1=a1=aic1=0;
      }                  
  
      if(length(c2)>=3){
         k2=order[2];
         cc2=c2[which(c2>k2)];    
         y2=y[cc2];
         x2=y[cc2-1];
         if(k2>1){
           for(j in 2:k2){
             x2=cbind(x2,y[cc2-j]);
           }
         }
       
         a2=coef(lm(y2~x2));
         re2=sum(resid(lm(y2~x2))^2);
         resid2=resid(lm(y2~x2));
         aic2=length(y2)*log(re2/length(y2))+2*(k2+1);
       }
     
       if(length(c2)<3) { re2=a2=k2=aic2=0;}
          if(length(c34)>=3){
            k34=order[3];
            cc34=c3[which(c34>k34)];
            y34=y[cc34];
            x34=y[cc34-1];
            if(k34>1){
              for(j in 2:k34){
                x34=cbind(x34,y[cc34-j]);
              }
            }
            
            a34=coef(lm(y34~x34));
            re34=sum(resid(lm(y34~x34))^2);
            resid34=resid(lm(y34~x34));
            aic34=length(y34)*log(re34/length(y34))+2*(k34+1);
          }
        
          if(length(c34)<3) { re34=k34=a34=aic34=0;}
                   
          re=(re1+re2+re34);
          maic=(aic1+aic2+aic34);
          if(re<minre){
            r1min=rr1[r1];
            r2min=rr2[r2];
            minre=re;
            mina1=a1;
            mina2=a2;
            mina34=a34;
            aic=maic;
          }
     }
  }
  threshold=c(r1min,r2min);
  result<-list(a1=mina1,a2=mina2,a34=mina34,threshold=threshold,rss=minre,aic=aic)
  return(result);
}


if(sum(regime==c(1,1,2,2))==4){
for(r1 in t1:t2){
  for(r2 in t3:t4){
    c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
    c2=which(z1<rr1[r1]&z2>=rr2[r2]);
    c3=which(z1<rr1[r1]&z2<rr2[r2]);
    c4=which(z1>=rr1[r1]&z2<rr2[r2]);
    c12=c(c1,c2);
    c34=c(c3,c4);
    if(length(c12)>=3){
      k12=order[1];
      cc12=c1[which(c12>k12)];
      y12=y[cc12];
      x12=y[cc12-1];
      if(k12>1){
        for(j in 2:k12){
          x12=cbind(x12,y[cc12-j])
        }
      }
      
      a12=coef(lm(y12~x12));
      re12=sum(resid(lm(y1~x1))^2);
      resid12=resid(lm(y12~x12));
      aic12=length(y12)*log(re12/length(y12))+2*(k12+1);
    }
    
    if(length(c12)<3){ 
      re12=a12=aic12=0;
    }                  
                      
    if(length(c34)>=3){
      k34=order[2];
      cc34=c3[which(c34>k34)];
      y34=y[cc34];
      x34=y[cc34-1];
      if(k34>1){
        for(j in 2:k34){
          x34=cbind(x34,y[cc34-j]);
        }
      }
      
      a34=coef(lm(y34~x34));
      re34=sum(resid(lm(y34~x34))^2);
      resid34=resid(lm(y34~x34));
      aic34=length(y34)*log(re34/length(y34))+2*(k34+1);
    }
    
    if(length(c34)<3) { re34=k34=a34=aic34=0;}
                    
    re=(re12+re34);
    maic=(aic12+aic34);
    if(re<minre){
      r1min=rr1[r1];
      r2min=rr2[r2];
      minre=re;
      mina12=a12;
      mina34=a34;
      aic=maic;
    }            
  }
 }
 threshold=c(r1min,r2min);
 result<-list(a12=mina12,a34=mina34,threshold=threshold,rss=minre,aic=aic)
 return(result);
}

if(sum(regime==c(1,2,1,2))==4){
for(r1 in t1:t2){
  for(r2 in t3:t4){
    c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
    c2=which(z1<rr1[r1]&z2>=rr2[r2]);
    c3=which(z1<rr1[r1]&z2<rr2[r2]);
    c4=which(z1>=rr1[r1]&z2<rr2[r2]);
    c13=c(c1,c3);
    c24=c(c2,c4);

    if(length(c13)>=3){
      k13=order[1];
      cc13=c13[which(c13>k13)];
      y13=y[cc13];
      x13=y[cc13-1];
      if(k13>1){
        for(j in 2:k13){
          x13=cbind(x13,y[cc13-j])
        }
      }
      
      a13=coef(lm(y13~x13));
      re13=sum(resid(lm(y13~x13))^2);
      resid13=resid(lm(y13~x13));
      aic13=length(y13)*log(re13/length(y13))+2*(k13+1);
    }
   
    if(length(c13)<3){ re13=a13=aic13=0;}
      if(length(c24)>=3){
        k24=order[2];
        cc24=c2[which(c24>k24)];    
        y24=y[cc24];
        x24=y[cc24-1];
        if(k24>1){
          for(j in 2:k24){
            x24=cbind(x24,y[cc24-j]);
          }
        }
        
        a24=coef(lm(y24~x24));
        re24=sum(resid(lm(y24~x24))^2);
        resid24=resid(lm(y24~x24));
        aic24=length(y24)*log(re24/length(y24))+2*(k24+1);
      }
      
      if(length(c24)<3) { re24=a24=k24=aic24=0;}    
        re=(re13+re24);
        maic=(aic13+aic24);
        if(re<minre){
          r1min=rr1[r1];
          r2min=rr2[r2];
          minre=re;
          mina13=a13;
          mina24=a24;
          aic=maic;
        }                 
     }
  }
  threshold=c(r1min,r2min);
  result<-list(a13=mina13,a24=mina24,threshold=threshold,rss=minre,aic=aic)
  return(result);
}

if(sum(regime==c(1,2,2,1))==4){
  for(r1 in t1:t2){
    for(r2 in t3:t4){
      c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
      c2=which(z1<rr1[r1]&z2>=rr2[r2]);
      c3=which(z1<rr1[r1]&z2<rr2[r2]);
      c4=which(z1>=rr1[r1]&z2<rr2[r2]);
      c14=c(c1,c4);
      c23=c(c2,c3);
      if(length(c14)>=3){
        k14=order[1];
        cc14=c14[which(c14>k14)];
        y14=y[cc14];
        x14=y[cc14-1];
        if(k14>1){
          for(j in 2:k14){
            x14=cbind(x14,y[cc14-j])
          }
        }
        
        a14=coef(lm(y14~x14));
        re14=sum(resid(lm(y14~x14))^2);
        resid14=resid(lm(y14~x14));
        aic14=length(y14)*log(re14/length(y14))+2*(k14+1);
      }
      
      if(length(c14)<3){ 
        re14=a14=aic14=0;}
        if(length(c23)>=3){
          k23=order[2];
          cc23=c23[which(c23>k23)];    
          y23=y[cc23];
          x23=y[cc23-1];
          if(k23>1){
            for(j in 2:k23){
              x23=cbind(x23,y[cc23-j]);
            }
        }
        
        a23=coef(lm(y23~x23));
        re23=sum(resid(lm(y23~x23))^2);
        resid23=resid(lm(y23~x23));
        aic23=length(y23)*log(re23/length(y23))+2*(k23+1);
      }
      
      if(length(c23)<3) { re23=a23=k23=aic23=0;}   
      
      re=(re14+re23);
      maic=(aic14+aic23);
      if(re<minre){
        r1min=rr1[r1];
        r2min=rr2[r2];
        minre=re;
        mina14=a14;
        mina23=a23;
        aic=maic;
      }                        
    }
  }
  threshold=c(r1min,r2min);
  result<-list(a14=mina14,a23=mina23,threshold=threshold,rss=minre,aic=aic)
  return(result);
}

if(sum(regime==c(1,1,1,2))==4){
  for(r1 in t1:t2){
    for(r2 in t3:t4){
      c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
      c2=which(z1<rr1[r1]&z2>=rr2[r2]);
      c3=which(z1<rr1[r1]&z2<rr2[r2]);
      c4=which(z1>=rr1[r1]&z2<rr2[r2]);
      c123=c(c1,c2,c3);
                       
      if(length(c123)>=3){
        k123=order[1];
        cc123=c123[which(c123>k123)];
        y123=y[cc123];
        x123=y[cc123-1];
        if(k123>1){
          for(j in 2:k123){
            x123=cbind(x123,y[cc123-j])
          }
        }
        
        a123=coef(lm(y123~x123));
        re123=sum(resid(lm(y123~x123))^2);
        resid123=resid(lm(y123~x123));
        aic123=length(y123)*log(re123/length(y123))+2*(k123+1);
      }
      
      if(length(c123)<3){re123=a123=aic123=0;}
      
      if(length(c4)>=3){
        k4=order[2];
        cc4=c4[which(c4>k4)];    
        y4=y[cc4];
        x4=y[cc4-1];
        if(k4>1){
           for(j in 2:k4){
             x4=cbind(x4,y[cc4-j]);
           }
        }
        
        a4=coef(lm(y4~x4));
        re4=sum(resid(lm(y4~x4))^2);
        resid4=resid(lm(y4~x4));
        aic4=length(y4)*log(re4/length(y4))+2*(k4+1);
      }
      
      if(length(c4)<3) { re4=a4=k4=aic4=0;}   
 
      re=(re123+re4);
      maic=(aic123+aic4);
      if(re<minre){
        r1min=rr1[r1];
        r2min=rr2[r2];
        minre=re;
        mina123=a123;
        mina4=a4;
        aic=maic;
      }                             
    }
  }
  threshold=c(r1min,r2min);
  result<-list(a123=mina123,a4=mina4,threshold=threshold,rss=minre,aic=aic)
  return(result);
}

if(sum(regime==c(1,2,1,1))==4){
  for(r1 in t1:t2){
    for(r2 in t3:t4){
      c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
      c2=which(z1<rr1[r1]&z2>=rr2[r2]);
      c3=which(z1<rr1[r1]&z2<rr2[r2]);
      c4=which(z1>=rr1[r1]&z2<rr2[r2]);
      c134=c(c1,c3,c4);
                          
      if(length(c134)>=3){
        k134=order[1];
        cc134=c134[which(c134>k134)];
        y134=y[cc134];
        x134=y[cc134-1];
        if(k134>1){
          for(j in 2:k134){
            x134=cbind(x134,y[cc134-j])
          }
        }
        
        a134=coef(lm(y134~x134));
        re134=sum(resid(lm(y134~x134))^2);
        resid134=resid(lm(y134~x134));
        aic134=length(y134)*log(re134/length(y134))+2*(k134+1);
      }
      
      if(length(c134)<3){ 
        re134=a134=aic134=0;}
        if(length(c2)>=3){
          k2=order[2];
          cc2=c2[which(c2>k2)];    
          y2=y[cc2];
          x2=y[cc2-1];
          if(k2>1){
            for(j in 2:k2){
              x2=cbind(x2,y[cc2-j]);
            }
          }
          
          a2=coef(lm(y2~x2));
          re2=sum(resid(lm(y2~x2))^2);
          resid2=resid(lm(y2~x2));
          aic2=length(y2)*log(re2/length(y2))+2*(k2+1);
        }
        
        if(length(c2)<3) { re2=a2=k2=aic2=0;}   
        
        re=(re134+re2);
        maic=(aic134+aic2);
        
        if(re<minre){
          r1min=rr1[r1];
          r2min=rr2[r2];
          minre=re;
          mina134=a134;
          mina2=a2;
          aic=maic;
        }                             
    }
  }
  threshold=c(r1min,r2min);
  result<-list(a134=mina134,a2=mina2,threshold=threshold,rss=minre,aic=aic)
  return(result);
}

if(sum(regime==c(1,1,2,1))==4){
for(r1 in t1:t2){
  for(r2 in t3:t4){
    c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
    c2=which(z1<rr1[r1]&z2>=rr2[r2]);
    c3=which(z1<rr1[r1]&z2<rr2[r2]);
    c4=which(z1>=rr1[r1]&z2<rr2[r2]);
    c124=c(c1,c2,c4);
    
    if(length(c124)>=3){
      k124=order[1];
      cc124=c124[which(c124>k124)];
      y124=y[cc124];
      x124=y[cc124-1];
      if(k124>1){
        for(j in 2:k124){
          x124=cbind(x124,y[cc124-j])
        }
      }
      
      a124=coef(lm(y124~x124));
      re124=sum(resid(lm(y124~x124))^2);
      resid124=resid(lm(y124~x124));
      aic124=length(y124)*log(re124/length(y124))+2*(k124+1);
    }

    if(length(c124)<3){re124=a124=aic124=0;}
    
    if(length(c3)>=3){
      k3=order[2];
      cc3=c3[which(c3>k3)];    
      y3=y[cc3];
      x3=y[cc3-1];
      if(k3>1){
        for(j in 2:k3){
          x3=cbind(x3,y[cc3-j]);
        }
      }
      
      a3=coef(lm(y3~x3));
      re3=sum(resid(lm(y3~x3))^2);
      resid3=resid(lm(y3~x3));
      aic3=length(y3)*log(re3/length(y3))+2*(k3+1);
    }
    
    if(length(c3)<3) { re3=a3=k3=aic3=0;} 
    
    re=(re124+re3);
    maic=(aic124+aic3);
    
    if(re<minre){
      r1min=rr1[r1];
      r2min=rr2[r2];
      minre=re;
      mina124=a124;
      mina3=a3;
      aic=maic;
    }                             
   }
 }
 threshold=c(r1min,r2min);
 result<-list(a124=mina124,a3=mina3,threshold=threshold,rss=minre,aic=aic)
 return(result);
}

if(sum(regime==c(1,2,2,2))==4){
  for(r1 in t1:t2){
    for(r2 in t3:t4){
      c1=which(z1>=rr1[r1]&z2>=rr2[r2]);
      c2=which(z1<=rr1[r1]&z2>=rr2[r2]);
      c3=which(z1<rr1[r1]&z2<rr2[r2]);
      c4=which(z1>=rr1[r1]&z2<rr2[r2]);
      c234=c(c2,c3,c4);
      
      if(length(c1)>=3){
        k1=order[1];
        cc1=c1[which(c1>k1)];
        y1=y[cc1];
        x1=y[cc1-1];
        if(k1>1){
          for(j in 2:k1){
            x1=cbind(x1,y[cc1-j])
          }
        }
        
        a1=coef(lm(y1~x1));
        re1=sum(resid(lm(y1~x1))^2);
        resid1=resid(lm(y1~x1));
        aic1=length(y1)*log(re1/length(y1))+2*(k1+1);
      }
      
      if(length(c1)<3){re1=a1=aic1=0;}
      
      if(length(c234)>=3){
        k234=order[2];
        cc234=c234[which(c234>k234)];    
        y234=y[cc234];
        x234=y[cc234-1];
        if(k234>1){
          for(j in 2:k234){
            x234=cbind(x234,y[cc234-j]);
          }
        }
        
        a234=coef(lm(y234~x234));
        re234=sum(resid(lm(y234~x234))^2);
        resid234=resid(lm(y234~x234));
        aic234=length(y234)*log(re234/length(y234))+2*(k234+1);
      }
      
      if(length(c234)<3) { re234=a234=k234=aic234=0;}
      
      re=(re1+re234);
      maic=(aic1+aic234);
      if(re<minre){
        r1min=rr1[r1];
        r2min=rr2[r2];
        minre=re;
        mina1=a1;
        mina234=a234;
        aic=maic;
      }
   }
 }
 threshold=c(r1min,r2min);
 result<-list(a1=mina1,a234=mina234,threshold=threshold,rss=minre,aic=aic);
 return(result);
}

if(sum(regime==c(1,1,1,1))==4){
  c1234=1:ly;
  k1234=order;
  cc1234=c1234[which(c1234>k1234)];
  x1234=y[cc1234-k1234];
  if(k1234>1){
    for(j in 2:k1234){
      x1234=cbind(x1234,y[cc1234-j])
    }
  }
  
  a1234=coef(lm(y1234~x1234));
  re1234=sum(resid(lm(y1234~x1234))^2);
  resid1234=resid(lm(y1234~x1234));
  aic1234=length(y1234)*log(re1234/length(y1234))+2*(k1234+1);
  threshold=c(0,0);
  result<-list(a1234=a1234,threshold=threshold,rss=re1234,aic=aic1234);
  return(result);
}

}  # end function


